/*
 ##############################################################
 # (C) 2014 BAE SYSTEMS
 #
 # ECOA Bish Bash Bosh Sample - Reusable BLOB (binary large object) class
 #
 # $Id: BLOB.c,v 1.2 2017/11/06 15:33:39 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "BLOB.h"

#include <string.h>
#include <stdio.h>

struct BLOB_t {
	size_t size;
	void *data;
};

size_t encode( void *dest, const void *src, size_t size )
{
	size_t i;
	void *d = dest;
	for( i = 0; i < size; i++ ){
		d += sprintf( d, "%x", *((unsigned char*)(src+i)) );
	}
	*(char*)d = '\0';
	return ( d - dest );
}

BLOB blobCreate( void *data, size_t size )
{
	BLOB blob;
	//
	blob = malloc( sizeof( struct BLOB_t ) );
	if( blob && size ){
		blob->data = malloc( 2*size );
		memset( blob->data, '\0', 2*size );
		//
		blob->size = encode( blob->data, data, size );
	}
	return blob;
}

void blobDestroy( BLOB blob )
{
	if( blob ){
		if( blob->data ) free( blob->data );
		free( blob );
	}
}

char* blobToString( BLOB blob, char *buff )
{
	sprintf( buff, "#%ld#%s#", (long)blob->size, (char*)blob->data );
	return buff;
}
