/*
 * @file BishBashBosh_modA_Im.c
 * Module Interface for Module BishBashBosh_modA_Im
 */
/*
 ##############################################################
 # (C) 2016 BAE SYSTEMS
 #
 # ECOA Bish Bash Bosh Sample
 #
 # $Id: BishBashBosh_modA_Im.c,v 1.5 2017/11/06 15:33:39 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/

#include "BishBashBosh_modA_Im.h"

#include "bbb.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../BishBashBosh_modA_Im/BLOB.h"

static bbb__LogItem LOGITEM = (bbb__LogItem){ 0, { 0, 0 }, { 3, { 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0, 10.0 }}};

void BishBashBosh_modA_Im__INITIALIZE__received(BishBashBosh_modA_Im__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "BishBashBosh_modA_Im__INITIALIZE__received" );
//	BishBashBosh_modA_Im_container__log_info( context, msg );
	//
	context->user.seq = 0;
	context->user.TDEL = (ECOA__duration){ 1, 0 };
	context->user.count = -1;
}

void BishBashBosh_modA_Im__START__received(BishBashBosh_modA_Im__context *context)
{
	ECOA__log msg;
	ECOA__global_time firstTime;
	//
	msg.current_size = sprintf( msg.data, "BishBashBosh_modA_Im__START__received" );
//	BishBashBosh_modA_Im_container__log_info( context, msg );
	//
	BishBashBosh_modA_Im_container__get_UTC_time( context, &(context->user.start_time) );
	//
	// Start the ball rolling...
	firstTime.seconds     = context->user.start_time.seconds + 1;
	firstTime.nanoseconds = context->user.start_time.nanoseconds;
	BishBashBosh_modA_Im_container__Trig1Set__send( context, &firstTime, 1234, &LOGITEM );
	BishBashBosh_modA_Im_container__Trig2Set__send( context, &firstTime );
	BishBashBosh_modA_Im_container__Trig3Set__send( context, &firstTime, 1234, &LOGITEM );
	//
	// This will fail, 'cos Trig4 is provided by a Module (not a DynTrig) - and that (probably) won't be running yet...
	// So see Tick() instead.
	BishBashBosh_modA_Im_container__Trig4Set__send( context, &firstTime, 1234, &LOGITEM );
}

void BishBashBosh_modA_Im__STOP__received(BishBashBosh_modA_Im__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "BishBashBosh_modA_Im__STOP__received" );
	BishBashBosh_modA_Im_container__log_info( context, msg );
}

void BishBashBosh_modA_Im__SHUTDOWN__received(BishBashBosh_modA_Im__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "BishBashBosh_modA_Im__SHUTDOWN__received" );
	BishBashBosh_modA_Im_container__log_info( context, msg );
}

ECOA__double64 ReadSensor( int sensor )
{
	return (double)rand()/(double)(sensor+1);
}

void BishBashBosh_modA_Im__Tick__received(BishBashBosh_modA_Im__context *context)
{
	ECOA__global_time utc_time, t4Time;
	bbb__LogItem      logEntry;
	BLOB              readingsBLOB;
	ECOA__double64    reading[10];
	ECOA__int32       sensor;
	ECOA__log         msg;
	FILE             *log = fopen( "data.log", "a" );
	//
	BishBashBosh_modA_Im_container__get_UTC_time( context, &utc_time );
	LOGITEM.timeStamp.secs  = utc_time.seconds     - (context->user.start_time.seconds);
	LOGITEM.timeStamp.nsecs = utc_time.nanoseconds - (context->user.start_time.nanoseconds);
	//
	// We should now be able to start the Trig4 - as the providing Module will now be running...
	// But of course we only want to kick it once...
	if( context->user.count < 0 ){
		t4Time.seconds = utc_time.seconds = 1;
		t4Time.nanoseconds = utc_time.nanoseconds;
		BishBashBosh_modA_Im_container__Trig4Set__send( context, &t4Time, 1234, &LOGITEM );
	}
	//
	for( sensor = 0; sensor < 10; sensor++ ){
		reading[sensor] = ReadSensor( sensor );
	}
	readingsBLOB = blobCreate( reading, sizeof( reading ));
	//
	logEntry.timeStamp.secs  = utc_time.seconds;
	logEntry.timeStamp.nsecs = utc_time.nanoseconds;
	logEntry.seq = ++(context->user.seq);
	blobToString( readingsBLOB, logEntry.data.data );
	logEntry.data.current_size = strlen( logEntry.data.data );
	//
	if( log ) {
		fprintf( log,    "%ld:%ld.%09ld:%s\n", (long)logEntry.seq, (long)logEntry.timeStamp.secs, (long)logEntry.timeStamp.nsecs, logEntry.data.data );
//		fprintf( stdout, "%ld:%ld.%09ld:%s\n", (long)logEntry.seq, (long)logEntry.timeStamp.secs, (long)logEntry.timeStamp.nsecs, logEntry.data.data );
	}
	if( context->user.count++ == 20 ){
		BishBashBosh_modA_Im_container__Trig2Reset__send( context );
		msg.current_size = sprintf( msg.data, "BishBashBosh_modA_Im_container__Trig2Reset__send() sent\007" );
		BishBashBosh_modA_Im_container__log_info( context, msg );
		context->user.count = 0;
	}
	if( context->user.count == 5 ){
		BishBashBosh_modA_Im_container__Trig2Set__send( context, &(ECOA__global_time){ utc_time.seconds+10, utc_time.nanoseconds } );
		BishBashBosh_modA_Im_container__Trig2Set__send( context, &(ECOA__global_time){ utc_time.seconds+11, utc_time.nanoseconds } );
		BishBashBosh_modA_Im_container__Trig2Set__send( context, &(ECOA__global_time){ utc_time.seconds+12, utc_time.nanoseconds } );
		BishBashBosh_modA_Im_container__Trig2Set__send( context, &(ECOA__global_time){ utc_time.seconds+13, utc_time.nanoseconds } );
	}
	//
	blobDestroy( readingsBLOB );
	fclose( log );
}

void BishBashBosh_modA_Im__Bish__received(BishBashBosh_modA_Im__context *context,
    const ECOA__int32 a,
    const bbb__LogItem *b)
{
	ECOA__global_time next_time;
	ECOA__log msg;
	//
	BishBashBosh_modA_Im_container__get_UTC_time( context, &next_time );
	context->user.TDEL.seconds = ( context->user.TDEL.seconds + 1 ) % 10;
	msg.current_size = sprintf( msg.data, "Bish! (DL_Trig1) a = %u; b->timeStamp = {%u,%09u}; next in %u.%09u seconds...", a, b->timeStamp.secs, b->timeStamp.nsecs, context->user.TDEL.seconds, context->user.TDEL.nanoseconds );
	BishBashBosh_modA_Im_container__log_info( context, msg );
	//
	next_time.seconds     += context->user.TDEL.seconds;
	next_time.nanoseconds += context->user.TDEL.nanoseconds;
	BishBashBosh_modA_Im_container__Trig1Set__send( context, &next_time, 1234, &LOGITEM );
}

void BishBashBosh_modA_Im__Bash__received(BishBashBosh_modA_Im__context *context)
{
	ECOA__global_time next_time;
	ECOA__log msg;
	//
	BishBashBosh_modA_Im_container__get_UTC_time( context, &next_time );
	context->user.TDEL.seconds = ( context->user.TDEL.seconds + 2 ) % 30;
	msg.current_size = sprintf( msg.data, "Bash! (DL_Trig2) next in %u.%09u seconds...", context->user.TDEL.seconds, context->user.TDEL.nanoseconds );
	BishBashBosh_modA_Im_container__log_info( context, msg );
	//
	next_time.seconds     += context->user.TDEL.seconds;
	next_time.nanoseconds += context->user.TDEL.nanoseconds;
	BishBashBosh_modA_Im_container__Trig2Set__send( context, &next_time );
}

void BishBashBosh_modA_Im__Bosh__received(BishBashBosh_modA_Im__context *context,
    const ECOA__int32 a,
    const bbb__LogItem *b)
{
	ECOA__global_time next_time;
	ECOA__log msg;
	//
	BishBashBosh_modA_Im_container__get_UTC_time( context, &next_time );
	context->user.TDEL.seconds = ( context->user.TDEL.seconds + 3 ) % 30;
	msg.current_size = sprintf( msg.data, "Bosh! (DL_Trig3) a = %d; b->timeStamp = {%u,%09u}; next in %u.%09u seconds...", a, b->timeStamp.secs, b->timeStamp.nsecs, context->user.TDEL.seconds, context->user.TDEL.nanoseconds );
	BishBashBosh_modA_Im_container__log_info( context, msg );
	//
	next_time.seconds     += context->user.TDEL.seconds;
	next_time.nanoseconds += context->user.TDEL.nanoseconds;
	BishBashBosh_modA_Im_container__Trig3Set__send( context, &next_time, 1234, &LOGITEM );
	BishBashBosh_modA_Im_container__Trig4Set__send( context, &next_time, 1234, &LOGITEM );
}

