/*
 ##############################################################
 # (C) 2014 BAE SYSTEMS
 #
 # ECOA Bish Bash Bosh Sample - Reusable ring buffer class
 #
 # $Id: Ring.c,v 1.2 2017/11/06 15:33:39 ksappleb Exp $
 #
 # Part of the BAE Systems, Rochester, ECOA Samples collection
 #
 # Created by K S Appleby, BAE Systems, Airport Works, Rochester, Kent
 #
 # This software is developed for and on behalf of BAE Systems (Operations)
 # Limited, Dassault Aviation, Bull SAS, Thales Systemes Aeroportes,
 # GE Aviation Systems Limited,
 # General Dynamics United Kingdom Limited and Leonardo MW Ltd,
 # and the copyright is owned by BAE Systems (Operations) Limited,
 # Dassault Aviation, Bull SAS, Thales Systemes Aeroportes, 
 # GE Aviation Systems Limited, General Dynamics United Kingdom
 # Limited and Leonardo MW Ltd
 #
 # This software is developed by BAE Systems (Operations)
 # Limited, Electronic Systems, and is the Intellectual
 # Property of BAE Systems (Operations) Limited, Electronic Systems.
 #
 # The information set out in this document is provided solely
 # on an 'as is' basis and the co-developers of this software make
 # no warranties expressed or implied, including no warranties
 # as to completeness, accuracy or fitness for purpose, with respect
 # to any of the information.
 ##############################################################
*/
#include "Ring.h"

#include <string.h>

struct Ring_t {
	size_t itemSize;
	size_t ringLength;
	void *first;
	void *next;
	void *last;
};

Ring ringCreate( size_t ringLength, size_t itemSize )
{
	Ring ring = malloc( sizeof( struct Ring_t ));
	ring->itemSize = itemSize;
	ring->ringLength = ringLength;
	ring->next = ring->last = ring->first = malloc( ringLength * itemSize );
	//
	return ring;
}

void ringDestroy( Ring ring )
{
	;
}

int ringAddItem( Ring ring, void *item )
{
	memcpy( ring->next, item, ring->itemSize );
	ring->next += ring->itemSize;
	if( ring->next > ring->first + ring->ringLength * ring->itemSize ){
		ring->next = ring->first;
	}
	if( ring->next == ring->last ){
		ring->last += ring->itemSize;
		if( ring->last > ring->first + ring->ringLength * ring->itemSize ){
			ring->last = ring->first;
		}
	}
	return 0;
}

void* ringGetItem( Ring ring )
{
	void *item = 0;
	//
	if( ring->last != ring->next ){
		item = malloc( ring->itemSize );
		memcpy( item, ring->last, ring->itemSize );
		ring->last += ring->itemSize;
		if( ring->last > ring->first + ring->ringLength * ring->itemSize ){
			ring->last = ring->first;
		}
	}
	return item;
}

int ringClear( Ring ring )
{
	ring->next = ring->last = ring->first;
	return 0;
}
