/* File dynTrig_im.c */

#include "dynTrig_im.h"

#include <stdio.h>  /* for NULL */
#include <stdlib.h> /* for malloc & free */
#include <unistd.h> /* for usleep() */
#include <string.h> /* for memcpy() */
#include <pthread.h>
#include <osl004_apos.h> /* BLEURGH! Needed for Create_Thread() */

#if defined( VXW )
static unsigned usleep( int x )
{
	struct timespec PERIOD, RES;
	PERIOD.tv_nsec = ( x % 1000000 ) * 1000;
	PERIOD.tv_sec = x / 1000000;
	nanosleep( &PERIOD, &RES );
	/**/
	return ((RES.tv_sec * 1000000) + (RES.tv_nsec / 1000));
}
#endif

void dynTrig_im__INITIALIZE__received(dynTrig_im__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "dynTrig_im__INITIALIZE__received" );
//	dynTrig_im_container__log_info( context, msg );
}

void dynTrig_im__START__received(dynTrig_im__context *context)
{
 	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "dynTrig_im__START__received" );
//	dynTrig_im_container__log_info( context, msg );
}

void dynTrig_im__STOP__received(dynTrig_im__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "dynTrig_im__STOP__received" );
//	dynTrig_im_container__log_info( context, msg );
}

void dynTrig_im__SHUTDOWN__received(dynTrig_im__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "dynTrig_im__SHUTDOWN__received" );
//	dynTrig_im_container__log_info( context, msg );
}

/*
 ****************************************************************************************************************************************
 * This is an implementation in ECOA long-hand for a dynamic trigger module.
 *
 * It works by explicitly spawning a thread that sits waiting for the delay period, and then shoots off the "out" Event..
 *
 ****************************************************************************************************************************************
 * It CANNOT be used in the general case because it explicitly KNOWS what the additional parameters to the "in" and "out" operations are,
 * namely "a" and "b".
 ****************************************************************************************************************************************
 */
typedef struct thrdInfo_t {
	dynTrig_im__context *context;
	unsigned usec;
	ECOA__int32 a;
	bbb__LogItem b;
} thrdInfo;

static void waitFunc(void* p)
{
	thrdInfo *thrdInf = (thrdInfo *)p;
	//
	usleep( thrdInf->usec );
	//
	dynTrig_im_container__out__send( thrdInf->context, thrdInf->a, &(thrdInf->b) );
	//
	free( p );
}

void dynTrig_im__in__received(dynTrig_im__context *context,
    const ECOA__global_time *expiryTime,
    const ECOA__int32 a,
    const bbb__LogItem *b)
{
	ECOA__global_time now;
	ECOA__duration    delayTime;
	ECOA__log         msg;
	thrdInfo         *thrdInf = NULL;
	Thread_Attr_Type  thrdAttr;
	Create_Thread_Status_Type ctStat;
	int               thrdId;
	//
	dynTrig_im_container__get_UTC_time( context, &now );
	delayTime.seconds = expiryTime->seconds - now.seconds;
	delayTime.nanoseconds = expiryTime->nanoseconds - now.nanoseconds;
	//
	msg.current_size = sprintf( msg.data, "dynTrig_im__in__received for a %g second wait", (double)(delayTime.seconds) + ((double)(delayTime.nanoseconds) * 1.0e-9 ));
//	dynTrig_im_container__log_info( context, msg );
	//
	if( delayTime.seconds >= 0 && delayTime.nanoseconds > 0 ){
		thrdInf = (thrdInfo*)malloc( sizeof( thrdInfo ));
		thrdInf->context = context;
		thrdInf->usec = (delayTime.seconds * 1000000 ) + (delayTime.nanoseconds / 1000 );
		thrdInf->a = a;
		memcpy( &(thrdInf->b), b, sizeof(bbb__LogItem));
		//
		thrdAttr = (Thread_Attr_Type){ Sched_FIFO, 99, 1000000 }; // 99 =max priority Linux...
		Create_Thread( &thrdAttr, &waitFunc, thrdInf, &thrdId, &ctStat );
	}
}

void dynTrig_im__reset__received(dynTrig_im__context *context)
{
	ECOA__log msg;
	//
	msg.current_size = sprintf( msg.data, "dynTrig_im__reset__received" );
//	dynTrig_im_container__log_info( context, msg );
}

